// Cansat7
// code by jph

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import java.lang.*;
import javax.swing.*;

class Cansat7
extends Remote
{
	static final Random myRandom=new Random(System.currentTimeMillis());
	static final int MOD=512;
	static final int MAX=16384;
	static final int ACC=4;
	static final int ALR=8;
	static final int GSV=16;
	static final int MSG=1;
	static final int WAI=2;	
	static final int FRAME_LEN=42;
	static final int FRAME_GPS=11;
	static final int GSV_LEN=55;	
	
	static final String[] STATS={"last contact","message type","most recent","spec request","supply","hardware","gps satellites","est. altitude","system clock"};
	static final String[] SWITC={"Accelerometer","Satellites in View","Buzzer"};
	static final String[] HEADS={"GPS Time","Latitude [deg]","Longitude [deg]","Temperature [C]","Pressure [kPa]"};
	static final String[] HEADA={"x-Accel. [g]","y-Accel. [g]","z-Accel. [g]"};
	
	static final String[] TEMPS={"inside","outside"};
	static final Color[]  TEMPC={Color.red,Color.orange};
	static final String[] PRESS={""};
	static final Color[]  PRESC={Color.blue};
	static final String[] ACCAS={""};
	static final Color[]  ACCAC={Color.magenta};	

	static final int[]   acc_mean={297,385,325};
	static final float[] acc_scale={0.00615385f,-0.00606061f,0.00606061f};
	
	static final int TIME=0;	
	static final int GPSY=1;
	static final int GPSX=2;	
	static final int TOUT=3;
	static final int PRES=4;
	static final int ACCX=5;
  static final int ACCY=6;
  static final int ACCZ=7;

 	static final int SID=0;
	static final int SEL=1;
	static final int SAZ=2;
	static final int SDB=3;
  
  JFrame myJFrame;
  Container myContainer;
    
  JTabbedPane myJTabbedPane;
	JTable myJTable;
  GpsMonitor myGpsMonitor;
  DataMonitor myDataMonitor[];
  EarthMonitor myEarthMonitor;

 	static final int COL_O=0;
	static final int COL_G=1;
	static final int COL_Y=2;
	static final int COL_R=3;
  
  static final int pMAREA=0;
 	static final int pSYCTL=1;
 	static final int pSTATS=2;
 	static final int pSUPLY=3;
	static final int pSWITC=4;
	static final int pCRRNT=5;
	static final int pNUTAT=6;
	static final int pACCEL=7;
	static final int pCREDI=8;
	static final int pEARTH=9;
	JLabel[] myJLabel;
	JTextField[] myJTextField;
	JSlider myJSlider;
	static final int tSTATS=0;
	static final int tLASTC=0;
	static final int tMTYPE=1;		
	static final int tMOSTR=2;
	static final int tSPECR=3;
	static final int tSUPLY=4;
	static final int tHARDW=5;
	static final int tGPSTS=6;		
	static final int tALTID=7;
	static final int tCSYNC=8;
	static final int tCRRNT=9;
	static final int tHEADA=14;
	ImageIcon[] myImageIcon;	
	JToggleButton myJToggleButton[];
	static final int tACCEL=0;
	static final int tGPGSV=1;	
	static final int tALARM=2;	
	
	Runtime myRuntime;
	
	BufferedWriter myBufferedWriter;
	BufferedWriter myFileWriter;
	long last_manifest;
	
	Cansat7(Terminal myTerminal) {  	
  	this.myTerminal=myTerminal;   	 	
		int c1;
		myRuntime=Runtime.getRuntime();
		String myString;
  	myImageIcon=new ImageIcon[4];
 		myImageIcon[COL_O]=new ImageIcon("res/icon.bbz.gif");
  	myImageIcon[COL_G]=new ImageIcon("res/icon.bbg.gif");
  	myImageIcon[COL_Y]=new ImageIcon("res/icon.bby.gif");
  	myImageIcon[COL_R]=new ImageIcon("res/icon.bbr.gif");
		myJFrame=new JFrame("Cansat 7");
  	myContainer=new Container();
  	myContainer.setLayout(new BorderLayout());
	  myJTabbedPane=new JTabbedPane();
	    JPanel[] myJPanel=new JPanel[10];   	  
   	  myJPanel[pMAREA]=new JPanel();
   	  myJPanel[pMAREA].setLayout(new BorderLayout());
   	    myJPanel[pSYCTL]=new JPanel();
   	    myJPanel[pSYCTL].setLayout(new FlowLayout());
   	    myJPanel[pSTATS]=new JPanel();
	      myJPanel[pSTATS].setBorder(BorderFactory.createEtchedBorder());
	      myJPanel[pSTATS].setLayout(new GridLayout(STATS.length,2,5,5));
	      myJLabel=new JLabel[STATS.length+HEADS.length];
	      myJTextField=new JTextField[STATS.length+HEADS.length+HEADA.length];
	      for (c1=0;c1<STATS.length;c1++) {
  	      myJLabel[tSTATS+c1]=new JLabel(STATS[c1],myImageIcon[COL_O],JLabel.LEFT);
  	      myJPanel[pSTATS].add(myJLabel[tSTATS+c1]);
   	      myJTextField[tSTATS+c1]=new JTextField("");
   	      myJPanel[pSTATS].add(myJTextField[tSTATS+c1]);
 	      }
 	      myJPanel[pSYCTL].add(myJPanel[pSTATS]);
			  myJPanel[pSYCTL].add(new JLabel("   "));
		    myJPanel[pSWITC]=new JPanel();
	      myJPanel[pSWITC].setBorder(BorderFactory.createEtchedBorder());
	      myJPanel[pSWITC].setLayout(new GridLayout(SWITC.length,1,5,5));	  	
     	  myJToggleButton=new JToggleButton[SWITC.length];
     	  for (c1=0;c1<SWITC.length;c1++) {
     	    myJToggleButton[c1]=new JToggleButton(SWITC[c1]);
     	    myJToggleButton[c1].setIcon(myImageIcon[COL_Y]);     	    	
     	    myJPanel[pSWITC].add(myJToggleButton[c1]);
   	    }
     	  myJToggleButton[tACCEL].setSelected(true);
     	  myJToggleButton[tGPGSV].setSelected(true);
     	  myJPanel[pSYCTL].add(myJPanel[pSWITC]);
     	  myJPanel[pSYCTL].add(new JLabel("   "));
     	  myJPanel[pCRRNT]=new JPanel();
	      myJPanel[pCRRNT].setBorder(BorderFactory.createEtchedBorder());
	      myJPanel[pCRRNT].setLayout(new GridLayout(HEADS.length+HEADA.length,2,5,5));
	      for (c1=0;c1<HEADS.length;c1++) {
  	      myJPanel[pCRRNT].add(new JLabel(HEADS[c1]));
   	      myJTextField[tCRRNT+c1]=new JTextField("");
   	      myJPanel[pCRRNT].add(myJTextField[tCRRNT+c1]);
	      }
	      for (c1=0;c1<HEADA.length;c1++) {
  	      myJPanel[pCRRNT].add(new JLabel(HEADA[c1]));
   	      myJTextField[tHEADA+c1]=new JTextField("");
   	      myJPanel[pCRRNT].add(myJTextField[tHEADA+c1]);
	      }
     	  myJPanel[pSYCTL].add(myJPanel[pCRRNT]);
     	  myJPanel[pSYCTL].add(new JLabel("   "));
   	    myJPanel[pNUTAT]=new JPanel();
   	    myJPanel[pNUTAT].setBorder(BorderFactory.createEtchedBorder());
   	    myJPanel[pNUTAT].setLayout(new BorderLayout());
   	    
   	    myJSlider=new JSlider(JSlider.VERTICAL,-180,0,0);
	  		myJSlider.setMajorTickSpacing(30);
				myJSlider.setMinorTickSpacing(10);
				myJSlider.setPaintTicks(true);
				myJSlider.setPaintLabels(true);
				myJSlider.setSnapToTicks(false);				   	    
				myJSlider.setEnabled(false);
				myJPanel[pNUTAT].add("North",new JLabel("Nutation [deg]"));
   	    myJPanel[pNUTAT].add("Center",myJSlider);
   	    myJPanel[pSYCTL].add(myJPanel[pNUTAT]);

     	myJPanel[pMAREA].add("Center",myJPanel[pSYCTL]);     	
     	  myJPanel[pCREDI]=new JPanel();
    		myJPanel[pCREDI].setBorder(BorderFactory.createEtchedBorder());
    		myJPanel[pCREDI].setLayout(new BorderLayout());
    		myJPanel[pCREDI].add("West",new JLabel(" ",new ImageIcon("res/group.jpg"),JLabel.CENTER));
    		JTextArea myJTextArea=new JTextArea("The objectives of the CanSat workshop is to develop, design, implement, and test a standalone system, which can monitor specific atmospheric properties during a descent in a fairly unknown environment. The acquired data is to be sent via radiolink to a ground station. The project includes the development of the descending probe as well as the implementation of the associated ground station. The photo shows the members of group 7, f.l.t.r: Angel Mario Cano, Jan Philipp Hakenberg, Narayanan Krishnamurthy, Lars Anders Forslund, Hankang Wang, Pakasit Worracharoen.");
	    	myJTextArea.setLineWrap(true);
    		myJTextArea.setWrapStyleWord(true);
    		myJPanel[pCREDI].add("Center",myJTextArea);
    	myJPanel[pMAREA].add("South",myJPanel[pCREDI]);
    	myJPanel[pMAREA].add("North",new JLabel(" ")); // spaceholder
   	myJTabbedPane.addTab("Mission surveillance",new ImageIcon("res/icon.hln.gif"),myJPanel[pMAREA]);		
		  myJTable=new JTable(new String[MOD][HEADS.length],HEADS);
		  myJTable.setRowHeight(20);
		  myJTable.setIntercellSpacing(new Dimension(2,2));
		myJTabbedPane.addTab("Atmospheric data",new ImageIcon("res/icon.dte.gif"),new JScrollPane(myJTable));	
			myGpsMonitor=new GpsMonitor();
			myGpsMonitor.setMinimumSize(new Dimension(200,200));
			myGpsMonitor.setPreferredSize(new Dimension(400,500));
			myGpsMonitor.mode=Monitor.MANUAL_TIMING;
			myDataMonitor=new DataMonitor[3];    	
    	myDataMonitor[0]=new DataMonitor("Temperature C",TEMPS,TEMPC);
    	myDataMonitor[0].setMargin(1,1);
    	myDataMonitor[0].setTimers(300,20000);
    	myDataMonitor[0].mode=Monitor.MANUAL_TIMING;
    	myDataMonitor[1]=new DataMonitor("Pressure kPa"  ,PRESS,PRESC);
    	myDataMonitor[1].setMargin(1,1);    	
    	myDataMonitor[1].setTimers(300,20000);
    	myDataMonitor[1].mode=Monitor.MANUAL_TIMING;
    	myDataMonitor[2]=new DataMonitor("Acceleration g",ACCAS,ACCAC);
    	myDataMonitor[2].setMargin(1,1);
    	//myDataMonitor[2].setMargin(.5f,1);
    	myDataMonitor[2].setTimers(30,3000);
    	myDataMonitor[2].redraw=60;
    	for (c1=0;c1<3;c1++) {
     	  myDataMonitor[c1].setMinimumSize(new Dimension(100,100));
    	  myDataMonitor[c1].setPreferredSize(new Dimension(100,130));
  	  }
    	JSplitPane myJSplitPaneH=new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
    	JSplitPane myJSplitPaneV=new JSplitPane(JSplitPane.VERTICAL_SPLIT);
    	JSplitPane myJSplitPaneW=new JSplitPane(JSplitPane.VERTICAL_SPLIT);
    	myJSplitPaneH.setLeftComponent(myGpsMonitor);
    	myJSplitPaneH.setRightComponent(myJSplitPaneV);    	
    	myJSplitPaneH.setOneTouchExpandable(true);
    	myJSplitPaneH.setContinuousLayout(true);
    	myJSplitPaneV.setTopComponent   (myDataMonitor[0]);
    	myJSplitPaneV.setBottomComponent(myJSplitPaneW);
    	myJSplitPaneW.setTopComponent   (myDataMonitor[1]);
    	myJSplitPaneW.setBottomComponent(myDataMonitor[2]);    	
    	myJSplitPaneV.setOneTouchExpandable(true);
    	myJSplitPaneV.setContinuousLayout(true);
   	myJTabbedPane.addTab("Monitoring of data",new ImageIcon("res/icon.bar.gif"),myJSplitPaneH);
   	  //myJPanel[pEARTH]=new JPanel();
   	  myEarthMonitor=new EarthMonitor();   	   	  
   	myJTabbedPane.addTab("Satellites in view",new ImageIcon("res/icon.lop.gif"),myEarthMonitor);
		myContainer.add("Center",myJTabbedPane);
		myJFrame.getContentPane().add(myContainer);
		myJFrame.setLocation(300,300);
    myJFrame.setSize(800,495+15);
    myJFrame.setVisible(true); 
    last_manifest=System.currentTimeMillis();   
	  myThread=new Thread(this);	  
	  myThread.start();
	}
	
	byte idf=0;
	int req=0;
	long thisrun; // updated every loop
	public void run() {
		int c1,c2=0,cnt=0;
		int ptr,clr,pnt=0;
		int[] myByte;
		float nutat=0;		
		float altid=100;
		int msg,xor,cnv;
		long cur_time=System.currentTimeMillis(); // updated by gps		
		long clock=cur_time; // system clock updated every new gps frame
		double cur_gpsx=0;
		double cur_gpsy=0;
		boolean myBoolean=false;
		String gps;
		String myString="",timeString;
		Calendar myCalendar=new GregorianCalendar();    
    myCalendar.setTime(new Date());		
		myTerminal.debug=0;	
		
		int max_msg=-1;
		int[] myInt=new int[MAX];  	
		
  	try {
		  myBufferedWriter=new BufferedWriter(new FileWriter("log/cansat7_timer"+"_"+clock+".txt"));
	  } catch (IOException myIOException) {
  	  System.out.println(myIOException);
	  }
  	try {	  
    	myFileWriter=new BufferedWriter(new FileWriter("log/cansat7_"+clock+".csv"));
	  } catch (IOException myIOException) {
  	  System.out.println(myIOException);
	  }
		while (myTerminal.isLaunched) {
			try {
				myThread.sleep(5);
			} catch (InterruptedException myInterruptedException) {
				System.out.println(myInterruptedException);
			}
  		thisrun=System.currentTimeMillis();
		  long time=cur_time+(thisrun-clock);
		  myGpsMonitor.time=time;
		  myDataMonitor[0].time=time;
		  myDataMonitor[1].time=time;

  		while (myInt[req]>0||req+105<max_msg)
  		  req++;  		  		  
		  if (myInt[req+1]==0)
  		  idf=2;
		  else
		    idf=1;
		  if (req>max_msg)
		    idf=0;
      myJTextField[tLASTC].setText((thisrun-last_manifest)+" ms");      
		  if (last_manifest+5000<thisrun)
		    myJLabel[tLASTC].setIcon(myImageIcon[COL_R]);
  	  else
		  if (last_manifest+2000<thisrun)
		    myJLabel[tLASTC].setIcon(myImageIcon[COL_Y]);
      else
        myJLabel[tLASTC].setIcon(myImageIcon[COL_G]);		  
			myByte=myTerminal.read();	
			ptr=0;
			clr=0;
			while (ptr+5<myByte.length) { 
// DOWNLINK: acceleration
// [10] C7A xor axL axH ayL ayH azL azH
  			if (ptr+10<=myByte.length)
			    if (myByte[ptr+0]=='C'&&myByte[ptr+1]=='7'&&myByte[ptr+2]=='A') {
  			    manifest("C7A");
  			    xor=0;
  				  for (c1=3;c1<9;c1++) // check for coherent
  				    xor^=myByte[ptr+c1];
  				  if (myByte[ptr+9]==xor) {
  			      float[] acc=new float[4];
  			      for (c1=0;c1<3;c1++) {
  			        cnv=myByte[ptr+3+c1*2]+(myByte[ptr+4+c1*2]<<8);
  			        acc[c1]=(cnv-acc_mean[c1])*acc_scale[c1];
  			        acc[3]+=acc[c1]*acc[c1];
  			        myString=""+acc[c1];
  			        if (myString.length()>1)
  			          if (myString.charAt(0)!='-')
  			            myString=" "+myString;
  			        if (myString.length()>6)
  			          myString=myString.substring(0,7);
         	      myJTextField[tHEADA+c1].setText(myString);         	      
			        }
			        acc[3]=(float)Math.sqrt(acc[3]);
			        myDataMonitor[2].add(0,System.currentTimeMillis(),acc[3]);
			        nutat*=.8;
			        nutat-=57.2958f*Math.acos(acc[1]/acc[3])*.2f;
			        myJSlider.setValue((int)nutat);
		        }
  			    clr=Math.max(clr,ptr+=9);
			    }
// DOWNLINK: gsv
// [60] C7V nums gsv56
  			if (ptr+3+GSV_LEN<=myByte.length)
			    if (myByte[ptr+0]=='C'&&myByte[ptr+1]=='7'&&myByte[ptr+2]=='V') {
  			    manifest("C7V");
  			    String gsv="";
  			    int[] myTok=new int[4];
  			    c1=3;
  			    while ('X'!=(char)myByte[ptr+c1]&&c1<3+GSV_LEN) {
  			      gsv+=(char)myByte[ptr+c1];
  			      if (myByte[ptr+c1]==',')
  			        gsv+="0";
  			      c1++;
			      }
  			    StringTokenizer myStringTokenizer=new StringTokenizer(gsv,",",false);
  			    try {    			    
    			    myStringTokenizer.nextToken();
  			      while (myStringTokenizer.hasMoreTokens()) {    			      
    			      for (c1=0;c1<4;c1++) {
      			      myTok[c1]=Integer.parseInt(myStringTokenizer.nextToken());      			      
    			      }
    			      if (myTok[3]>0) {
    			        //System.out.println("sat# "+myTok[0]+" elev="+myTok[1]+" azim="+myTok[2]+" dB="+myTok[3]);    			        
    			        double lat=0.0174533*cur_gpsy;
    			        double latCos=Math.cos(lat);
    			        double latSin=Math.sin(lat);
    			        double lon=0.0174533*cur_gpsx;    			      
    			        double lonCos=Math.cos(lon);
    			        double lonSin=Math.sin(lon);
    			        double e=0.0174533*(90-myTok[SEL]);    			      
    			        double eCos=Math.cos(e);
    			        double eSin=Math.sin(e);    			      
    			        double a=0.0174533*myTok[SAZ];
    			        double aCos=Math.cos(a);
    			        double aSin=Math.sin(a);
    			        double d=.5*(-12756.3*eSin+53200*Math.sqrt(0.942506*eCos*eCos+eSin*eSin))/(eCos*eCos+eSin*eSin);
    			        double x1=latCos*lonCos*(6378.14+d*eSin)-d*eCos*lonCos*aSin*latSin-d*aCos*eCos*lonSin;
                  double x2=d*aCos*eCos*lonCos+latCos*(6378.14+d*eSin)*lonSin-d*eCos*aSin*latSin*lonSin;
                  double x3=d*eCos*latCos*aSin+(6378.14+d*eSin)*latSin;
    			        double gpsx=57.2958*Math.atan2(x2,x1);
    			        double gpsy=57.2958*Math.atan2(x3,Math.sqrt(x1*x1+x2*x2));
    			        //System.out.println(" \\- lat +"+gpsy+" lon "+gpsx);
    			        myEarthMonitor.add(
    			          thisrun+7000,
    			          myTok[SID],
    			          gpsx,
    			          gpsy,
    			          ""+myTok[SID]+" "+myTok[SEL]+" "+myTok[SAZ]+" "+myTok[SDB]);
  			        }
			        }
			      } catch (Exception myException) {
  			      System.out.println("corrupt:"+gsv);
			      }
  			    clr=Math.max(clr,ptr+=3+GSV_LEN-2);
			    }
// DOWNLINK: listener
// [ 7] C7L portB supplyL supplyH xor
  			if (ptr+7<=myByte.length)
			    if (myByte[ptr+0]=='C'&&myByte[ptr+1]=='7'&&myByte[ptr+2]=='L') {  			    
  			    manifest("C7L");
  			    c7m();  			    
  			    xor=0;
  				  for (c1=3;c1<6;c1++)
  				    xor^=myByte[ptr+c1];
  				  if (xor==myByte[ptr+6]) {
   			      if ((myByte[ptr+3]&ACC)==0) 
    			      myJToggleButton[tACCEL].setIcon(myImageIcon[COL_O]);
  			      else
  			        myJToggleButton[tACCEL].setIcon(myImageIcon[COL_G]);
   			      if ((myByte[ptr+3]&GSV)==0) 
    			      myJToggleButton[tGPGSV].setIcon(myImageIcon[COL_O]);
  			      else
  			        myJToggleButton[tGPGSV].setIcon(myImageIcon[COL_G]);
  			      if ((myByte[ptr+3]&ALR)==0) 
    			      myJToggleButton[tALARM].setIcon(myImageIcon[COL_O]);
  			      else
  			        myJToggleButton[tALARM].setIcon(myImageIcon[COL_G]);  			      

  			      cnv=myByte[ptr+4]+(myByte[ptr+5]<<8);
  			      float volts=cnv*0.023042f;
  			      myString=""+volts;
  			      if (myString.length()>6)
  			      myString=myString.substring(0,6);
  			      myJTextField[tSUPLY].setText(myString+" V");
  			      if (volts<7)
  			        myJLabel[tSUPLY].setIcon(myImageIcon[COL_R]);
  			      else
  			      if (volts<8)
  			        myJLabel[tSUPLY].setIcon(myImageIcon[COL_Y]);
  			      else
  			        myJLabel[tSUPLY].setIcon(myImageIcon[COL_G]);  			      
			      }
			      clr=Math.max(clr,ptr+=6);
		      }
// DOWNLINK: cansat
// [42] C7S msgL msgH t0L t0H t1L t1H prL prH GPS xor
			  if (ptr+FRAME_LEN<=myByte.length)
				  if (myByte[ptr+0]=='C'&&myByte[ptr+1]=='7'&&myByte[ptr+2]=='S') {
  				  manifest("C7S");
  				  xor=0;
  				  for (c1=3;c1<FRAME_LEN-1;c1++)
  				    xor^=myByte[ptr+c1];
  				  if (xor==myByte[ptr+FRAME_LEN-1]) {    				  
 				  		float pres=0,tins=0,tout=0;
					    msg=myByte[ptr+3]+(myByte[ptr+4]<<8);
					    myBoolean=msg>max_msg; // tells us if most recent message
				      max_msg=Math.max(msg,max_msg);
				      myJTextField[tMOSTR].setText(""+max_msg);
					    if (0==myInt[msg]++) {
  					    //\begin{temperature}
  					      tins=temperature(myByte[ptr+5],myByte[ptr+6]);					        
					        if (myBoolean) {
					          myJTextField[tHARDW].setText(tins+" C");	
               		  if (55<tins)
              		    myJLabel[tHARDW].setIcon(myImageIcon[COL_R]);
                	  else
              		  if (40<tins)
              		    myJLabel[tHARDW].setIcon(myImageIcon[COL_Y]);
                    else
                      myJLabel[tHARDW].setIcon(myImageIcon[COL_G]);					     		  					     		  					          				        
				          }					          
					        tout=temperature(myByte[ptr+7],myByte[ptr+8]);
					        myJTable.setValueAt(""+tout,msg%MOD,TOUT);
					        if (myBoolean)
					          myJTextField[tCRRNT+TOUT].setText(""+tout);
					      //\end{temperature}
					      //\begin{pressure}
  					      cnv=myByte[ptr+9]+(myByte[ptr+10]<<8);
  					      cnv=5*(2432+25*cnv);
  					      pres=(float)cnv;
  					      pres/=1152f;
  					      myJTable.setValueAt(""+pres,msg%MOD,PRES);
  					      if (myBoolean) {
    					      myJTextField[tCRRNT+PRES].setText(""+pres);    					      
       			        altid*=.7;
	      		        altid+=altitude(pres,tout)*.3f;
  					        myJTextField[tSTATS+tALTID].setText((int)+altid+" m");
					        }  					      
					      //\end{pressure}
					      //\begin{gps}
  					      gps="";
	  				      for (c1=11;c1<FRAME_LEN-1;c1++)
		  			      	gps+=(char)myByte[ptr+c1];					        
					        try { // catch parse errors
					          long gps_time;
					          double gpsx=0,gpsy=0;
					          int sats=0;
					     		  myCalendar.set(Calendar.HOUR_OF_DAY,Integer.parseInt(gps.substring(0,2)));
					     		  myCalendar.set(Calendar.MINUTE     ,Integer.parseInt(gps.substring(2,4)));
					     		  myCalendar.set(Calendar.SECOND     ,Integer.parseInt(gps.substring(4,6)));
					     		  myCalendar.set(Calendar.MILLISECOND,Integer.parseInt(gps.substring(6,9)));
 					     		  myCalendar.add(Calendar.HOUR_OF_DAY,1);
				     		    gps_time=myCalendar.getTime().getTime();
				     		    myCalendar.add(Calendar.HOUR_OF_DAY,-1);
					     		  if (myBoolean) {
  				     		    cur_time=gps_time;
  				     		    clock=System.currentTimeMillis();
  				     		    long delay=(thisrun-cur_time)/1000;
  				     		    if (delay>0)
  		     		          myJTextField[tCSYNC].setText("+"+delay+" sec");
  		     		        else
  		     		          myJTextField[tCSYNC].setText(""+delay+" sec");  		     		          
  		     		        delay=Math.abs(delay);
                		  if (300<delay)
                		    myJLabel[tCSYNC].setIcon(myImageIcon[COL_R]);
                  	  else
                		  if (60<delay)
                		    myJLabel[tCSYNC].setIcon(myImageIcon[COL_Y]);
                      else
                        myJLabel[tCSYNC].setIcon(myImageIcon[COL_G]);					     		  					     		  
                    }                    
					     		  timeString=gps.substring(0,2)+":"+gps.substring(2,4)+":"+gps.substring(4,6)+"."+gps.substring(6,9);
					     			myJTable.setValueAt(timeString,msg%MOD,TIME);					     			
  					        if (myBoolean)
    					        myJTextField[tCRRNT+TIME].setText(""+timeString);
  					        sats=Integer.parseInt(gps.substring(28,30));
  					        if (myBoolean) {
	  				          myJTextField[tGPSTS].setText(""+sats);
	  				          myGpsMonitor.label=""+sats;
                		  if (sats<3)
                		    myJLabel[tGPSTS].setIcon(myImageIcon[COL_R]);
                  	  else
                		  if (sats<6)
                		    myJLabel[tGPSTS].setIcon(myImageIcon[COL_Y]);
                      else
                        myJLabel[tGPSTS].setIcon(myImageIcon[COL_G]);
  				          }
					     			gpsy=Double.parseDouble(gps.substring(9,13)+"."+gps.substring(13,17))/100.;					     			
					     			myString=(float)gpsy+" "+gps.charAt(17);
					     			if (sats<3)
					     			  myString="("+myString+")";
					     			myJTable.setValueAt(myString,msg%MOD,GPSY);
					     			if (gps.charAt(17)=='S')
					     			  gpsy*=-1;
  					        if (myBoolean) {
    					        cur_gpsy=gpsy;
	  				          myJTextField[tCRRNT+GPSY].setText(""+myString);
  				          }
					     			gpsx=Double.parseDouble(gps.substring(18,23)+"."+gps.substring(23,27))/100.;
					     			myString=(float)gpsx+" "+gps.charAt(27);
					     			if (sats<3)
					     			  myString="("+myString+")";					     			
					     			myJTable.setValueAt(myString,msg%MOD,GPSX);
					     			if (gps.charAt(27)=='W')
					     			  gpsx*=-1;
  					        if (myBoolean) {
    					        cur_gpsx=gpsx;
	  				          myJTextField[tCRRNT+GPSX].setText(""+myString);
	  				          myEarthMonitor.add(clock+10000,0,cur_gpsx,cur_gpsy,"");
  				          }
					          myGpsMonitor.add(gps_time,gpsx,gpsy);					          
					          myDataMonitor[0].add(0,gps_time,tins);
					          myDataMonitor[0].add(1,gps_time,tout);
					          myDataMonitor[1].add(0,gps_time,pres);
					          myFileWriter.write(""+msg+","+timeString+","+gpsx+","+gpsy+","+tout+","+pres+","+tins+"\n");
					      } catch (Exception myException) {
  					      System.out.println("message corrupt");
  				      }					      					    
			        }					    
					    clr=Math.max(clr,ptr+=FRAME_LEN-1);
				    }
				  }
		    ptr++;
		  }
		  myTerminal.clear(clr);
		}
		try {
  		myBufferedWriter.flush();
  		myBufferedWriter.close();
    } catch (IOException myIOException) {
     	  System.out.println(myIOException);
    }
		try {
  		myFileWriter.flush();
  		myFileWriter.close();
		} catch (IOException myIOException) {
     	  System.out.println(myIOException);
    }
  }
  
// UPLINK: mission control
// [8] C7M portB ident reqL reqH xor  
  void c7m() {
    int c1;
    byte[] upLink=new byte[8];
    upLink[0]='C';
    upLink[1]='7';
    upLink[2]='M';
    upLink[3]=0;    
 		if (myJToggleButton[tACCEL].isSelected())
 		  upLink[3]|=ACC;
 		if (myJToggleButton[tGPGSV].isSelected())
 		  upLink[3]|=GSV;
 		if (myJToggleButton[tALARM].isSelected())
 		  upLink[3]|=ALR;
 		upLink[4]=idf;
 		upLink[5]=(byte)(req&255);
 		upLink[6]=(byte)(req>> 8);
 		upLink[7]=0;
    for (c1=3;c1<7;c1++)
      upLink[7]^=upLink[c1];
    myTerminal.write(upLink);
    if (idf>0) {
      myJTextField[tSPECR].setText(""+req+"|"+idf);
      myJLabel[tSPECR].setIcon(myImageIcon[COL_Y]);
    } else {
      myJTextField[tSPECR].setText("none");
      myJLabel[tSPECR].setIcon(myImageIcon[COL_G]);
    }
    manifest("C7M");
  }
  
  float temperature(int tempL,int tempH) {
    int myInt=tempL+(tempH<<8);
    if (myInt>32768)
      myInt-=65536;
    return myInt/10f;
  }
  
  float altitude(float pres,float temp) {
    temp+=273.15f;
  	return -287.04f*temp/9.82f*(float)Math.log(pres/101.325);	
  } 
  
  void manifest(String myString) {
    myJTextField[tMTYPE].setText(myString);
	  try {
     	myBufferedWriter.write(myString+" "+(thisrun-myTerminal.atLaunch)+" "+(thisrun-last_manifest)+"\n");
     	last_manifest=System.currentTimeMillis();
 	  } catch (IOException myIOException) {
     	  System.out.println(myIOException);
    }
  }

  
}

