// DataMonitor
// code by jph

import java.lang.Math;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.text.*;
import java.util.*;

class Stamp
implements Comparator
{
  long  time;
  float data;
  
  Stamp() {}
  
  Stamp(long time) {
    this.time=time;
  }
  
  Stamp(long time,float data) {
    this.time=time;
    this.data=data;
  }
  
  public int compare(Object myObject0,Object myObject1) {    
    long myLong=((Stamp)myObject0).time-((Stamp)myObject1).time;
    if (myLong< 0) return -1;
    if (myLong==0) return  0;    
    return  1;
  }
}

class DataMonitor
extends Monitor
implements MouseListener,MouseMotionListener
{
	static final SimpleDateFormat mySimpleDateFormat=new SimpleDateFormat("hh:mm:ss",Locale.getDefault());
  static final int[] RESOLUTION={1,2,5,10,20,50,100,200,500,1000,2000,5000,10000,20000,50000,100000,200000,500000};	
	
	Pixel click;
	String[] myString;
	Color[] myColor;
		
	volatile TreeSet[] myTreeSet;
	int plots;
	
	double ptime;     //millis time per pixel
	long xtime;       //millis time interval between bars
	long ftime=1000;  //millis of future area
	long rtime,ltime;
	long otime=0;	    //millis offset
	double tdata,bdata,wdata;
	Pixel area;
	float marginB;
	float marginT;
	int countToClean;
	volatile boolean myBoolean;
		
	DataMonitor(String label,String[] myString,Color[] myColor) {
		super(label);
		int c1;
		this.myString=myString;
		this.myColor=myColor;
		plots=myString.length;
		myTreeSet=new TreeSet[plots];
		for (c1=0;c1<plots;c1++)
		  myTreeSet[c1]=new TreeSet(new Stamp());		
		marginB=0.1f;
		marginT=0.1f;
	  ptime=150;
	  xtime=10000;
	  countToClean=0;
	  myBoolean=false;
		addMouseListener(this);
		addMouseMotionListener(this);
	}
	
	void setTimers(double ptime,long xtime) {
  	this.ptime=ptime;
  	this.xtime=xtime;
	}
	
	void setMargin(float marginB,float marginT) {
  	this.marginB=marginB*100;
  	this.marginT=marginT*100;  	
	}
	
	void add(int myInt,long time,float data) {
  	while (myBoolean) ;
  	myTreeSet[myInt].add(new Stamp(time,data*100));
  	countToClean++;
	}
	
  void paintMonitor(Graphics myGraphics) {
	  int c1,c2;
	  long l1;
	  area=new Pixel(getSize().width-48,getSize().height-15);
   	rtime=time-otime;
		ltime=rtime-(long)(area.pix*ptime);
	  SortedSet[] mySortedSet=new SortedSet[plots];
	  Stamp[][] myStamp=new Stamp[plots][];
	  myBoolean=true;
	  if (countToClean>100) {
	    countToClean=0;
      for (c1=0;c1<plots;c1++) {
        mySortedSet[c1]=myTreeSet[c1].subSet(new Stamp(0),new Stamp(ltime-1000));
        myStamp[c1]=new Stamp[mySortedSet[c1].size()];
        mySortedSet[c1].toArray(myStamp[c1]);
        for (c2=0;c2<mySortedSet[c1].size();c2++)
          myTreeSet[c1].remove(myStamp[c1][c2]);
      }
    }
    for (c1=0;c1<plots;c1++) {      
      mySortedSet[c1]=myTreeSet[c1].subSet(new Stamp(ltime),new Stamp(rtime));
      myStamp[c1]=new Stamp[mySortedSet[c1].size()];      	      
      mySortedSet[c1].toArray(myStamp[c1]);
    }
    myBoolean=false;
		tdata=-10000.;
		bdata= 10000.;
		for (c1=0;c1<plots;c1++)
		  for (c2=0;c2<myStamp[c1].length;c2++) {
  		  bdata=Math.min(bdata,myStamp[c1][c2].data-marginB);
				tdata=Math.max(tdata,myStamp[c1][c2].data+marginT);				
			}
		//int gray;
		//gray=170;
  	//myGraphics.setColor(new Color(gray,gray,gray));		
  	myGraphics.setColor(Color.gray);
		myGraphics.fillRect(0,0,getSize().width,getSize().height);
		int gray;
		gray=220;
		myGraphics.setColor(new Color(gray,gray,gray));
		myGraphics.fillRect(0,0,area.pix,area.piy);
		//draw: yellow stripe of future second
		myGraphics.setColor(Color.yellow);
		myGraphics.fillRect(Xpixel(time),
                        0,
		                    (int)(ftime/ptime),
		                    getSize().height);
		//draw: vertical lines of
		for (l1=(ltime/xtime)*xtime;l1<rtime;l1+=xtime) {
			myGraphics.setColor(Color.gray);
			myGraphics.drawLine(Xpixel(l1),0,Xpixel(l1),area.piy);			
			EnhancedString(myGraphics,mySimpleDateFormat.format(new Date(l1)),Xpixel(l1)-27,area.piy+12);			
		}    
		//determine: resolution of horizontal lines
		int myInt=0;
		double res=(tdata-bdata)*15./(double)area.piy;
		for (c1=0;c1<RESOLUTION.length;c1++) {
			if (res<RESOLUTION[c1]) {
				myInt=c1;
				break;
			}
		}
		//draw: horizontal lines
		int grid_max_pixel;
		for (c1=(int)Math.ceil(bdata);c1<=(int)Math.floor(tdata);c1++)
			if ((c1%RESOLUTION[myInt])==0) {
				grid_max_pixel=(int)((tdata-c1)/(tdata-bdata)*area.piy);
				EnhancedString(myGraphics,Float.toString(c1/100f),area.pix+3,grid_max_pixel+5);
				myGraphics.setColor(Color.gray);
				if ((c1%RESOLUTION[myInt+1])==0)
					myGraphics.setColor(Color.darkGray);
				myGraphics.drawLine(0,grid_max_pixel,area.pix,grid_max_pixel);
			}
		//draw: lineStrip
		for (c1=0;c1<plots;c1++) {
  		EnhancedLineStrip(myColor[c1]);
			for (c2=0;c2<myStamp[c1].length;c2++)
    		EnhancedLineStrip(Xpixel(myStamp[c1][c2].time),
    		                  Ypixel(myStamp[c1][c2].data));
			EnhancedLineStrip(myGraphics);
		}    
   	//draw: axes and title
    myGraphics.setColor(Color.black);
		myGraphics.drawLine(0,area.piy,area.pix,area.piy);
		myGraphics.drawLine(area.pix,0,area.pix,area.piy);
    
		for (c1=0;c1<plots;c1++) {
			EnhancedLineStrip(myColor[c1]);
			EnhancedLineStrip(10,c1*13+22);
			EnhancedLineStrip(25,c1*13+22);
			EnhancedLineStrip(myGraphics);
			EnhancedString(myGraphics,myString[c1],30,c1*13+22+4);
		}		
  }
  
 	public int Xpixel(long myLong) {
	 	double myDouble0=myLong-ltime;
	 	double myDouble1=rtime-ltime;
	 	return (int)(area.pix*myDouble0/myDouble1);	 		 	
	}
	
 	public int Ypixel(double myDouble) {	  
	  double myDouble0=tdata-myDouble;
	  double myDouble1=tdata-bdata;
	  return (int)(area.piy*myDouble0/myDouble1);
 	}
 	
  public void mousePressed(MouseEvent myMouseEvent) {
	  //int c1;
		//click.pix=myMouseEvent.getX();
		//click.piy=myMouseEvent.getY();	  
	  //for (c1=0;c1<count;c1++) {
		//  if (Math.abs(lineStrip[c1].pix-click.pix)+
		//      Math.abs(lineStrip[c1].piy-click.piy)<10) {
		//		myCansat7.track[lineStrip[c1].idx].label=!myCansat7.track[lineStrip[c1].idx].label;
		//  }
	  //}
	  //super.repaint();
	}
	
	public void mouseReleased(MouseEvent myMouseEvent) {
		//if (myMouseEvent.getClickCount()==2) {
		//	otime=0;
		//	repaint();
		//}
	}
	
	public void mouseClicked(MouseEvent myMouseEvent) {};	
	
	public void mouseDragged(MouseEvent myMouseEvent) {
		//otime=otime+(long)((myMouseEvent.getX()-click.pix)*ptime);		
		//click.pix=myMouseEvent.getX();		
		//repaint();
	}
}
