// Monitor
// code by jph

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.text.*;
import java.util.*;

class Pixel {
	int pix;
	int piy;
	
	Pixel() {};
	
	Pixel(int pix,int piy) {
  	this.pix=pix;
  	this.piy=piy;
	}
	
	void set(int pix,int piy) {
  	this.pix=pix;
  	this.piy=piy;
	}
}

abstract class Monitor
extends JComponent
implements Runnable
{
  static final int AUTO_TIMING=0;
  static final int MANUAL_TIMING=1;
  
	String myString;
	Pixel mouse;
	int redraw;
	int mode;
	long time;
	Thread myThread;
	
	Monitor(String myString) {
		this.myString=myString;
		mouse=new Pixel();
		redraw=250;
		mode=0;
		time=System.currentTimeMillis();
		myThread=new Thread(this);
		myThread.start();
	}
	
	abstract void paintMonitor(Graphics myGraphics);
	
	public void paintComponent(Graphics myGraphics) {  	
  	if (mode==0)
  	  time=System.currentTimeMillis();
  	paintMonitor(myGraphics);
  	EnhancedString(myGraphics,myString,5,13);
	}
	
  public void run() {
  	while (true) {
			try {
				myThread.sleep(redraw);
			} catch (InterruptedException myInterruptedException) {
				System.out.println(myInterruptedException);
			}
      repaint();      
  	}
	}

  void EnhancedString(Graphics myGraphics,String myString,int x,int y) {
		myGraphics.setFont(new Font("verdana",Font.BOLD,11));
		myGraphics.setColor(Color.darkGray);
		myGraphics.drawString(myString,x+1,y+0);
		myGraphics.drawString(myString,x+0,y+1);
		myGraphics.setColor(Color.gray);
		myGraphics.drawString(myString,x-1,y+0);
		myGraphics.drawString(myString,x+0,y-1);		
		myGraphics.setColor(Color.white);
		myGraphics.drawString(myString,x,y);	
	}
	
  void TinyString(Graphics myGraphics,String myString,int x,int y) {
		myGraphics.setFont(new Font("verdana",Font.PLAIN,8));
		myGraphics.setColor(Color.white);
		myGraphics.drawString(myString,x,y);	
	}
	
	double arcAng=0;
	long arcSync=System.currentTimeMillis();
	void EnhancedArcedCircle(Graphics myGraphics,int x,int y) {		
		arcAng+=(double)(System.currentTimeMillis()-arcSync)*.07;
		arcSync=System.currentTimeMillis();
		myGraphics.setColor(Color.darkGray);
		myGraphics.fillArc(x-10,y-10,21,21,(int)arcAng+  0,40);
		myGraphics.fillArc(x-10,y-10,21,21,(int)arcAng+120,40);
		myGraphics.fillArc(x-10,y-10,21,21,(int)arcAng+240,40);
		myGraphics.setColor(Color.yellow);
		myGraphics.fillArc(x-11,y-11,21,21,(int)arcAng+  0,40);
		myGraphics.fillArc(x-11,y-11,21,21,(int)arcAng+120,40);
		myGraphics.fillArc(x-11,y-11,21,21,(int)arcAng+240,40);		
	}
	
	//partial implementation of MouseListener,MouseMotionListener
	public void mouseEntered(MouseEvent myMouseEvent) {
		//isMouseOver=true;
	}
	
	public void mouseExited(MouseEvent myMouseEvent) {
		//isMouseOver=false;		
	}
	
	public void mouseMoved(MouseEvent myMouseEvent) {
	  mouse.pix=myMouseEvent.getX();
	  mouse.piy=myMouseEvent.getY();		
	}	
	
	//-------------------------------------------------------------------------
	LinkedList myLinkedList;	
	Color myColor;
	void EnhancedLineStrip(Color myColor) {
		myLinkedList=new LinkedList();
		this.myColor=myColor;
	}
	
	void EnhancedLineStrip(int pix,int piy) {
  	myLinkedList.add(new Pixel(pix,piy));
	}
	
	void EnhancedLineStrip(Graphics myGraphics) {
		int c1;
		Pixel[] myPixel=new Pixel[myLinkedList.size()];
		myLinkedList.toArray(myPixel);
		myGraphics.setColor(Color.darkGray);
		for (c1=0;c1<myPixel.length-1;c1++) {
	    myGraphics.drawLine(myPixel[c1].pix+1,myPixel[c1].piy  ,myPixel[c1+1].pix+1,myPixel[c1+1].piy  );
	    myGraphics.drawLine(myPixel[c1].pix  ,myPixel[c1].piy+1,myPixel[c1+1].pix  ,myPixel[c1+1].piy+1);
   	}
    myGraphics.setColor(myColor);
    for (c1=0;c1<myPixel.length-1;c1++) {
	    myGraphics.drawLine(myPixel[c1].pix-1,myPixel[c1].piy  ,myPixel[c1+1].pix-1,myPixel[c1+1].piy  );
	    myGraphics.drawLine(myPixel[c1].pix  ,myPixel[c1].piy-1,myPixel[c1+1].pix  ,myPixel[c1+1].piy-1);
   	}
    for (c1=0;c1<myPixel.length-1;c1++)
 	    myGraphics.drawLine(myPixel[c1].pix  ,myPixel[c1].piy  ,myPixel[c1+1].pix  ,myPixel[c1+1].piy  );
 	  
 	  //int elev;
 		//for (c1=0;c1<count;c1++)
		//	if (myPixel[c1].myString) {				
		//		if (c1<count-1) {
		//			if (myPixel[c1+1].piy-myPixel[c1].piy>0)
		//				elev=-5;
		//			else
		//				elev=15;
		//			}
		//		else
		//			elev=5;				
		//		EnhancedString(myGraphics,myPixel[c1].str,myPixel[c1].pix+10,Math.max(myPixel[c1].piy+elev,10));
		//	}
	}	
}
